#!/usr/bin/perl 
###############################################################################
#   rpmdep.pl
#
#    Copyright (C) 2006 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id: rpmorphan-1.15 | rpmdep.pl |  | Eric Gerbier  $
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# this program search the full dependency of an installed rpm package
###############################################################################
use strict;
use warnings;

#use Data::Dump qw(dump);
use English '-no_match_vars';
use Getopt::Long;    # arg analysis
use Pod::Usage;      # man page

# library
## no critic (ProhibitNoisyQuotes)
use lib '/usr/lib/rpmorphan/';
use lib '.';
## no critic (RequireBarewordIncludes)
require 'rpmorphan-lib.pl'
  or die "problem to load rpmorphan-lib: $ERRNO\n";
## use critic

# the code should use no "print" calls
# but instead debug, warning, info calls
#########################################################
# is to be defined because rpmorphan-lib need it
# we just send this messages to debug
sub display_status($) {
	debug( shift @_ );
	return;
}
##########################################################
# resolv problems with some characters in names for graphwiz
sub norm($) {
	my $name = shift @_;

	## no critic (ProhibitEscapedMetacharacters)
	my $norm_name = $name;
	$norm_name =~ s/-/_/g;
	$norm_name =~ s/\./_/g;
	$norm_name =~ s/\+//g;

	return $norm_name;
}
#########################################################
sub is_package($$) {
	my $name        = shift @_;    # package name
	my $rh_provides = shift @_;    # general provide hash

	return exists $rh_provides->{$name};
}
#########################################################
# convert a name to a package name
sub name2pac($$$) {
	my $name           = shift @_;    # object name
	my $rh_is_provided = shift @_;    # general provide hash
	my $rh_provides    = shift @_;    # general provide hash

	my $pac;
	if ( is_package( $name, $rh_provides ) ) {

		# name is a package
		$pac = $name;
		debug("(name2pac) $name is a package");
	}
	elsif ( exists $rh_is_provided->{$name} ) {

		# $name is provided by a package
		$pac = $rh_is_provided->{$name};
		debug("(name2pac) $name is a file/virtual");
	}
	else {
		warning("(name2pac) can not find who provides $name");
		$pac = undef;
	}
	return $pac;
}
#########################################################
# search the dependency of $name
# no prototype because recursive
sub solve_dep {
	my $name           = shift @_;    # package name
	my $rh_is_provided = shift @_;    # general provide hash
	my $rh_provides    = shift @_;    # general provide hash
	my $rh_depends     = shift @_;    # general dependencies hash
	my $rh_files       = shift @_;    # general files hash
	my $rh_res         = shift @_;    # result cache
	my $fh_dot         = shift @_;    # flag for dot
	my $level          = shift @_;    # recurse level

	$level++;                         # recurse level
	my $state = 0;                    # state ( 0 for ok)

	foreach my $dep ( keys %{ $rh_depends->{$name} } ) {
		my $debug_txt = "(solve_dep $level) $name -> $dep ";

		if ( $dep =~ m/^rpmlib/ ) {

			# strange case, but real, for example :
			# rpmlib(PayloadFilesHavePrefix)
			# rpmlib(CompressedFileNames)
			debug( $debug_txt . ' (skip rpmlib)' );
			next;
		}

		# dep may be a package or a file or a virtual
		# searched package
		my $pac = name2pac( $dep, $rh_is_provided, $rh_provides );
		if ( !defined $pac ) {
			$state++;
			next;
		}
		else {
			$debug_txt .= " -> $pac ";
		}

		# filter dependency
		if ( $pac eq $name ) {

			# no dependency on self
			debug( $debug_txt . ' (not on self)' );
		}
		elsif ( exists $rh_res->{$name}{$pac} ) {

			# already found
			debug( $debug_txt . ' (already found)' );
		}
		else {

			# name depends on $pac
			# add $pac as relation
			$rh_res->{$name}{$pac} = 1;
			debug($debug_txt);
			if ($fh_dot) {
				print {$fh_dot} norm($name) . ' -> ' . norm($pac) . ";\n";
			}

			if ( !exists $rh_res->{$pac} ) {

				# recursive call on $pac
				my $sub_state =
				  solve_dep( $pac, $rh_is_provided, $rh_provides, $rh_depends,
					$rh_files, $rh_res, $fh_dot, $level );
				$state += $sub_state;
			}

			# add depends of $pac
			foreach my $d ( keys %{ $rh_res->{$pac} } ) {
				$rh_res->{$name}{$d} = 1;
			}
		}

	}    # foreach
	return $state;
}
#########################################################
# convert result for package in a human list
sub hres2human($$) {
	my $package = shift @_;
	my $h_res   = shift @_;

	if ( exists $h_res->{$package} ) {
		return join q{,}, sort keys %{ $h_res->{$package} };
	}
	else {
		return q{};
	}
}
#########################################################
# build reverse provide hash
sub build_provided($$) {
	my $rh_provides       = shift @_;    # input : general provide hash
	my $rh_is_provided_by = shift @_;    # output : provided hash

	# provides tells
	# $rh_provides->{$pac} : list files in package and meta
	# ex $rh_provides->{'perl'} : perl(Config), /usr/bin/perl
	#print "---------- perl provides -------------\n";
	#print dump $rh_provides->{'perl'};
	#print "-----------------------\n";
	foreach my $key1 ( keys %{$rh_provides} ) {
		foreach my $key2 ( @{ $rh_provides->{$key1} } ) {

			# key2 is provided by $key1 package
			$rh_is_provided_by->{$key2} = $key1;
		}
	}

	#print "---------- /usr/bin/perl is_provided_by -------------\n";
	#print dump $rh_is_provided_by->{'/usr/bin/perl'};
	#print "-----------------------\n";
	return;
}
#########################################################
# build reverse depends hash
sub build_depends_from($$) {
	my $rh_depends      = shift @_;    # input : general dependencies hash
	my $rh_depends_from = shift @_;    # output : general dependencies hash

	# depends hash means
	# rh_depends->{$pac) : list of depending packages of pac
	# ex $rh_depends->{'perl'} : afick, rpmorphan, rpmerizor ...
	# print "------ depending of perl -----------------\n";
	# print dump $rh_depends->{'perl'};
	#print "------ depending of /usr/bin/perl -----------------\n";
	#print dump $rh_depends->{'/usr/bin/perl'};

	# first we have to change order used %depends
	# we want to have :
	# $depends_from{$pac} : list of rpm pac dependencies
	# ex $depends_from{'rpmorphan'} : perl, logrotate
	foreach my $key1 ( keys %{$rh_depends} ) {
		foreach my $key2 ( keys %{ $rh_depends->{$key1} } ) {

			# ex key2 = afick-gui key1 = afick
			$rh_depends_from->{$key2}{$key1} = 1;
		}
	}

	#print "---------- rpmorphan depends_from -------------\n";
	#print dump $rh_depends_from->{'rpmorphan'};
	#print "-----------------------\n";
	return;
}
#########################################################
# convert dependencies on file/virtual to dependencies on package
sub convert2pac($$$) {
	my $rh_depends     = shift @_;    # input/output : general dependencies hash
	my $rh_provides    = shift @_;    # input : general provide hash
	my $rh_is_provided = shift @_;    # input : provided hash

	#print "------ depending of perl -----------------\n";
	#print dump $rh_depends->{'perl'};
	#print "------ depending of /usr/bin/perl -----------------\n";
	#print dump $rh_depends->{'/usr/bin/perl'};

	foreach my $name ( keys %{$rh_depends} ) {
		next if ( $name =~ m/^rpmlib/ );
		if ( !is_package( $name, $rh_provides ) ) {
			my $pac = name2pac( $name, $rh_is_provided, $rh_provides );
			next if ( !defined $pac );
			if ( exists $rh_depends->{$pac} ) {
				%{ $rh_depends->{$pac} } =
				  ( %{ $rh_depends->{$pac} }, %{ $rh_depends->{$name} } );
			}
			else {
				$rh_depends->{$pac} = $rh_depends->{$name};
			}
			delete $rh_depends->{$name};
		}
	}

	#print "------ depending of perl -----------------\n";
	#print dump $rh_depends->{'perl'};
	#print "------ depending of /usr/bin/perl -----------------\n";
	#print dump $rh_depends->{'/usr/bin/perl'};

	return;
}
#########################################################
#
#	main
#
#########################################################
my $version = '0.6';

my $opt_help;
my $opt_man;
my $opt_version;
my $opt_use_cache;
my $opt_clear_cache;
my $opt_verbose;
my $opt_dot;
my $opt_depending;
my $opt_all;

my %opt = (
	'help'        => \$opt_help,
	'man'         => \$opt_man,
	'verbose'     => \$opt_verbose,
	'version'     => \$opt_version,
	'dot'         => \$opt_dot,
	'use-cache'   => \$opt_use_cache,
	'clear-cache' => \$opt_clear_cache,
	'depending'   => \$opt_depending,
	'all'         => \$opt_all,
);

Getopt::Long::Configure('no_ignore_case');
GetOptions(
	\%opt,       'help|?', 'man',        'verbose',
	'version|V', 'dot=s',  'use-cache!', 'clear-cache',
	'depending', 'all|a',
) or pod2usage(2);

init_debug($opt_verbose);

if ($opt_help) {
	pod2usage(1);
}
elsif ($opt_man) {
	pod2usage( -verbose => 2 );
}
elsif ($opt_version) {
	print_version($version);
	exit;
}

# test if a target is set
if ( ( $#ARGV != 0 ) and not $opt_all ) {
	pod2usage('need a target : package name');
}
my $package = $ARGV[0];

# hash structures to be filled with rpm query
my %files;
my %install_time;
my %provides;
my %depends;
my %virtual;
my %requires;

read_rpm_data( \%opt, \%provides, \%install_time, \%files, \%depends, \%virtual,
	\%requires );

if ( ( not $opt_all ) and ( !exists $install_time{$package} ) ) {
	warning("package $package is not installed");
	exit 1;
}

my %is_provided_by;
build_provided( \%provides, \%is_provided_by );

convert2pac( \%depends, \%provides, \%is_provided_by );

my %depends_from;
build_depends_from( \%depends, \%depends_from );

# open dot graph ?
my $fh_dot;
if ($opt_dot) {

	# open output file
	## no critic (RequireBriefOpen)
	if ( open $fh_dot, '>', $opt_dot ) {
		print {$fh_dot} "digraph \"rpmdep\" {\n";
	}
	else {
		warning("can not write to $opt_dot file : $ERRNO\n");
		$opt_dot = 0;
	}
}

debug('2 : solve');

# store results in a hash $res{$pac}{$dep}
# so keys $res{$pac} is the result for $pac
my %res;
my $state = 0;    # for error status
if ($opt_all) {
	if ($opt_depending) {
		foreach my $pac ( keys %install_time ) {
			$state +=
			  solve_dep( $pac, \%is_provided_by, \%provides, \%depends, \%files,
				\%res, $fh_dot, 0 );
		}
	}
	else {
		foreach my $pac ( keys %install_time ) {
			$state +=
			  solve_dep( $pac, \%is_provided_by, \%provides, \%depends_from,
				\%files, \%res, $fh_dot, 0 );
		}
	}
	foreach my $pac (
		sort { scalar keys %{ $res{$a} } <=> scalar keys %{ $res{$b} } }
		keys %res
	  )
	{
		my $nb = scalar keys %{ $res{$pac} };
		my $res = hres2human( $pac, \%res );
		info("$nb $pac : $res");
	}
}
elsif ($opt_depending) {

	# search for depending packages
	#print "-- $package depends --\n";
	#print dump( $depends{$package} );
	$state =
	  solve_dep( $package, \%is_provided_by, \%provides, \%depends, \%files,
		\%res, $fh_dot, 0 );

	my $res = hres2human( $package, \%res );
	if ($res) {
		info("$package is needed by $res");
	}
	else {
		info("no depending : $package is an orphan");
	}
}
else {

	# search for dependencies
	#print "-- $package depends_from --\n";
	#print dump( $depends_from{$package} );
	$state = solve_dep( $package, \%is_provided_by, \%provides, \%depends_from,
		\%files, \%res, $fh_dot, 0 );

	my $res = hres2human( $package, \%res );
	if ($res) {
		info("$package depends upon $res");
	}
	else {
		info("$package has no dependencies");
	}
}

# close dot graph
if ($opt_dot) {
	print {$fh_dot} "}\n";
	close $fh_dot or die "problem to close dot file :$OS_ERROR\n";
}

exit $state;
__END__

=head1 NAME

rpmdep - display the full dependency of an installed rpm package

=head1 DESCRIPTION

rpmdep is an improved 'rpm -qR' command : 
it search recursively for package dependencies,
and resolvs all dependencies to package names.
It can also prepare a file to build a graph of dependencies, with graphviz.

=head1 SYNOPSIS

rpmdep.pl  [options] package

options:

   -help                brief help message
   -man                 full documentation
   -V, --version        print version
   -use-cache		use cache file instead rpm query
   -clear-cache		clear cache file

   -verbose             verbose
   -dot dotfile		build a dot file for graphviz
   -depending		instead of dependencies, display dependings
   -all			works on all installed packages

=head1 REQUIRED ARGUMENTS

a package to analyse. Only one name may be given.
It is not necessary if you are using the -all option.

=head1 OPTIONS

=over 8

=item B<-help>

Print a brief help message and exits.

=item B<-man>

Print the manual page and exits.

=item B<-version>

Print the program release and exit.

=item B<-verbose>

The program works and print debugging messages.

=item B<-dot>

create a dot file to be used by graphviz.
in normal call, an arrow from a to b means that a package depends (uses) b package.
in dependings call, an arrow from a to b means that a package is used by b package.

=item B<-use-cache>

the rpm query may be long (10 to 30 s). If you will run an rpmorphan tool
several time, this option will allow to gain a lot of time :
it save the rpm query on a file cache (first call), then
use this cache instead quering rpm (others calls).

=item B<-clear-cache>

to remove cache file. Can be used with -use-cache to write
a new cache.

=item B<--depending>

instead of showing package dependencies (rpm -qR),
display packages wich are depending upon the given argument :
it then act as an improved 'rpm -q --whatrequires' command

=item B<--all>

rpmdep will then work on all installed packages, and display the results
sorted by the number of depends (or dependings).

=back

=head1 USAGE

get the full list of dependencies for rpmrebuild package

  rpmdep.pl rpmrebuild

get the full list of dependencies for rpmrestore package and
build the graph data file (rpmrestore.dot)

  rpmdep.pl -dot rpmrestore.dot rpmrestore

build a png graph image from the graph data

  dot -Tpng rpmrestore.dot -o rpmrestore.png

get depending package of perl

  rpmdep.pl --depending perl

check is a package is an orphan

  rpmdep.pl --depending afick-gui

list all package dependencies

  rpmdep.pl -a

list all packages dependings and prepare a graph

  rpmdep.pl -a --depending --dot alldep.dot

=head1 FILES

the program can use the /tmp/rpmorphan.cache file

=head1 DIAGNOSTICS

the verbose mode allow to see all the recursive work

others messages are

=over 8

=item B<package ... is not installed>

this is not a name of an installed package

=item B<... has no dependencies>

this can comes for some admin tools (ash for example)

=item B<... depends upon (list)>

return a list of sorted and comma separated packages

=item B<... is needed by (list)>

return the depending package list

=item B<no depending : ... is an orphan>

the given package has no depending packages : it is an orphan

=back

=head1 EXIT STATUS

O if all is ok

>=1 if a problem

=head1 CONFIGURATION

nothing

=head1 DEPENDENCIES

you should use graphviz to build graph from 

=head1 INCOMPATIBILITIES

not known

=head1 BUGS AND LIMITATIONS

the program does not work well on program installed
in several versions

=head1 NOTES

this program can be used as "normal" user

=head1 SEE ALSO

=for man
\fIrpm\fR\|(1) for rpm call
.PP
\fIrpmorphan\fR\|(1)
.PP
\fIrpmusage\fR\|(1)
.PP
\fIrpmduplicates\fR\|(1)
.PP
\fIrpmextra\fR\|(1)

=for html
<a href="rpmorphan.1.html">rpmorphan(1)</a><br />
<a href="rpmusage.1.html">rpmusage(1)</a><br />
<a href="rpmduplicates.1.html">rpmduplicates(1)</a><br />
<a href="rpmextra.1.html">rpmextra(1)</a><br />

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 by Eric Gerbier
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

=head1 AUTHOR

Eric Gerbier

you can report any bug or suggest to gerbier@users.sourceforge.net

=cut

