#!/usr/bin/perl
###############################################################################
#
#    Copyright (C) 2006 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id: rpmorphan-1.15 | rpmduplicates.pl | Tue Nov 5 13:38:35 2013 +0100 | Eric Gerbier  $
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# search for packages with several versions installed
###############################################################################
use strict;
use warnings;

use English '-no_match_vars';

use Getopt::Long;    # arg analysis
use Pod::Usage;      # man page

# library
## no critic (ProhibitNoisyQuotes)
use lib '/usr/lib/rpmorphan/';
use lib '.';
## no critic (RequireBarewordIncludes)
require 'rpmorphan-lib.pl'
  or die "problem to load rpmorphan-lib: $ERRNO\n";
## use critic

my $VERSION = '0.4';

my $opt_help;
my $opt_man;
my $opt_version;
my $opt_verbose;

my %opt = (
	'help'    => \$opt_help,
	'man'     => \$opt_man,
	'verbose' => \$opt_verbose,
	'version' => \$opt_version,
);

Getopt::Long::Configure('no_ignore_case');
GetOptions( \%opt, 'help|?', 'man', 'verbose', 'version|V' )
  or pod2usage(2);

init_debug($opt_verbose);

if ($opt_help) {
	pod2usage(1);
}
elsif ($opt_man) {
	pod2usage( -verbose => 2 );
}
elsif ($opt_version) {
	print_version($VERSION);
	exit;
}

#  we cannot use common cache as duplicate package are not clean taken
#  by the common code
## no critic (RequireInterpolationOfMetachars)
my $cmd =
'rpm -qa --queryformat "%{NAME} %{VERSION}-%{RELEASE} %{ARCH} %{BUILDTIME}\n" ';
## use critic

my $fh;
## no critic (ProhibitTwoArgOpen,RequireBriefOpen)
if ( !open $fh, "$cmd |" ) {
	die "can not open $cmd : $ERRNO\n";
}
## use critic

my %h_ver;          # versions
my %h_buildtime;    # buildtime
my %h_arch;         # arch

my $nb_dup  = 0;
my $nb_arch = 0;
my $SPACE   = q{ };

while ( my $line = <$fh> ) {
	chomp $line;
	my ( $soft, $version, $arch, $build ) = split $SPACE, $line;

	next if ( $soft eq 'gpg-pubkey' );

	if ( exists $h_ver{$soft} ) {

		# we have another package with same name
		if ( $h_ver{$soft} eq $version ) {

			# same name, same version :
			# arch should be different
			print "info : found $soft $version on $arch and $h_arch{$soft}\n";
			$nb_arch++;
		}
		else {

			# we have already found a package with same name
			# but the version is different
			my $prev_ver   = $h_ver{$soft};
			my $prev_build = $h_buildtime{$soft};

			my $old;
			if ( $prev_build < $build ) {

				# current package was build after the previous stored one
				# keep the latest : the current
				$h_buildtime{$soft} = $build;

				# we will advice to delete the previous
				$old = $prev_ver;
			}
			else {

				# current package was build before the stored one
				# advice to delete the older (current)
				$old = $version;
			}

			# transform for human
			my $prev_date = localtime $prev_build;
			my $cur_date  = localtime $build;
			print
"duplicate $soft : $prev_ver (build on $prev_date) / $version (build on $cur_date)\n";
			print "suggest : rpm -e $soft-$old\n";
			$nb_dup++;
		}
	}
	else {
		$h_ver{$soft}       = $version;
		$h_buildtime{$soft} = $build;
		$h_arch{$soft}      = $arch;
	}
}
close $fh or warning("can not close $cmd : $ERRNO");

if ( $nb_dup == 0 ) {
	print "no duplicate found !\n";
}
else {
	print "$nb_dup rpm duplicate\n";
	print "$nb_arch arch duplicate\n";
}

exit $nb_dup;
__END__

=head1 NAME

rpmduplicates - find program with several version installed

=head1 DESCRIPTION

sometimes, the upgrade of a system will install new packages, and "forget" to remove
the older version. Rpmorphan do not work with this kind of unused packages. Rpmduplicates 
is the answer. I recommend to run it after each distribution upgrade 
(for example from fedora core 7 to 8).

=head1 SYNOPSIS

rpmduplicates.pl  [options]

options:

   -help                brief help message
   -man                 full documentation
   -V, --version        print version

   -verbose             verbose

=head1 REQUIRED ARGUMENTS

nothing

=head1 OPTIONS

=over 8

=item B<-help>

Print a brief help message and exits.

=item B<-man>

Print the full manual page and exits.

=item B<-version>

Print the program release and exit.

=item B<-verbose>

The program works and print debugging messages.

=back

=head1 USAGE

rpmduplicates.pl 

=head1 FILES

nothing

=head1 DIAGNOSTICS

it will show the duplicates with their version and build date

and suggest to remove the older one

example :

duplicate kernel-devel : 2.6.24.4-64.fc8 (Sat Mar 29 15:36:41 2008) / 2.6.24.3-50.fc8 (Thu Mar 20 20:43:47 2008)

suggest : rpm -e kernel-devel-2.6.24.3-50.fc8

=head1 EXIT STATUS

the number of duplicates packages found

=head1 CONFIGURATION

nothing

=head1 DEPENDENCIES

rpmorphan library

=head1 INCOMPATIBILITIES

not known

=head1 BUGS AND LIMITATIONS

not known

=head1 NOTES

this program can be used as "normal" user

the gpg-pubkey packages are excluded from the result

=head1 SEE ALSO

=for man
\fIrpm\fR\|(1) for rpm call
.PP
\fIrpmorphan\fR\|(1)
.PP
\fIrpmusage\fR\|(1)
.PP
\fIrpmdep\fR\|(1)
.PP
\fIrpmextra\fR\|(1)

=for html
<a href="rpmorphan.1.html">rpmorphan(1)</a><br />
<a href="rpmusage.1.html">rpmusage(1)</a><br />
<a href="rpmdep.1.html">rpmdep(1)</a><br />
<a href="rpmextra.1.html">rpmextra(1)</a><br />

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 by Eric Gerbier
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

=head1 AUTHOR

Eric Gerbier

you can report any bug or suggest to gerbier@users.sourceforge.net

=cut

